/* $Novell: /ldap/src/cldap/samples/cmpattrs.c,v 1.2 2000/08/21 22:59:04 vtag Exp $ */
/*************************************************************************

   Copyright 1999, 2000 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
    cpattrs.c
***************************************************************************
   Description: cpattrs.c does a compare operation to determine if 
                an entry contains particular objectClass and cn values.
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ldap.h>

static char usage[] =
"\n Usage:   cmpattrs <host name> <port number> <login dn> <password>"
" <compare dn>" 
"\n Example: cmpattrs acme.com 389 cn=admin,o=acme secret"
"\n\t  cn=james,ou=sales,o=acme\n";

void printout( int rc, char *dn, char *attr, char *vals );

int main( int argc, char **argv)
{ 

    int         version, ldapPort, rc;
    char        *ldapHost, *loginDN, *password;    
    char        *compareDN, *compareAttribute, *compareValue;       
    struct      berval bvalue;
    LDAP        *ld;

    if (argc != 6)
    {
        printf("%s", usage);
        exit ( 0 );
    }

    ldapHost    = argv[1];
    ldapPort    = atoi(argv[2]);
    loginDN     = argv[3];
    password    = argv[4];
    compareDN   = argv[5];

    /* Set LDAP version to 3 */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version);

    /* Initialize the LDAP session */
    if (( ld = ldap_init( ldapHost, ldapPort )) == NULL)
    {
        printf ( "\n\tLDAP session initialization failed\n");
        return( 1 );
    }
    printf("\n\tLDAP session initialized\n");

    /* Bind to the server */
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if (rc != LDAP_SUCCESS )
    {
        printf("ldap_simple_bind_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s ( ld );
        return( 1 );
    }
    printf("\n\tBind successful\n");


    /* 
     * Determine if this object contains the value "inetOrgPerson"
     * in its objectclass attribute. 
     */
    compareAttribute = "objectclass";
    compareValue = "inetOrgPerson";
    bvalue.bv_val = compareValue;
    bvalue.bv_len = strlen( compareValue );

    rc = ldap_compare_ext_s( ld,                /* LDAP session handle */
                             compareDN,         /* the object being compared */
                             compareAttribute,  /* the attribute to compare */
                             &bvalue,           /* the value to compare */
                             NULL,              /* server controls */
                             NULL);             /* client controls */

    printout( rc, compareDN, compareAttribute, compareValue );

    /* 
     * Determine if this object contains the value "Mary" in its cn attribute.
     */
    compareAttribute = "cn";
    compareValue = "Mary";
    bvalue.bv_val = compareValue;
    bvalue.bv_len = strlen( compareValue );

    rc = ldap_compare_ext_s( ld,                /* LDAP session handle */ 
                             compareDN,         /* the object being compared */
                             compareAttribute,  /* the attribute to compare */ 
                             &bvalue,           /* the value to compare */
                             NULL,              /* server controls */
                             NULL);             /* client controls */

    printout( rc, compareDN, compareAttribute, compareValue );

    ldap_unbind_s( ld );

    return( 0 );
}

void printout( int rc, char *dn, char *attr, char *vals )
{  
    switch ( rc )
    {
    case LDAP_COMPARE_TRUE:
        printf("\n\t%s has the value \"%s\"\n\t"
               "in the \"%s\" attribute.\n", dn, vals, attr );
        break;

    case LDAP_COMPARE_FALSE:
        printf("\n\t%s does NOT have the value \"%s\"\n\t"
               "in the \"%s\" attribute.\n", dn, vals, attr );
        break;

    default:
        printf("ldap_compare_ext_s: %s\n", ldap_err2string( rc ) );        
        break;
    }
}
