/* $Novell: /ldap/src/cldap/samples/delentry.c,v 1.4 2001/07/30 20:56:05 dsteck Exp $ */
/*************************************************************************

   Copyright 1999, 2000 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   delentry.c
***************************************************************************
   Description: The delentry.c sample deletes the entry with the 
                distinguished name provided by the user.
***************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <ldap.h>

static char usage[] =
"\n Usage:   delentry <host name> <port number> <login dn> <password>"
"\n\t  <deleteDN>" 
"\n Example: delentry acme.com 389 cn=admin,o=acme secret"
"\n\t  cn=james,ou=sales,o=acme\n";

int main( int argc, char **argv)
{ 

    int         version, ldapPort, rc; 
    char        *ldapHost, *loginDN, *password, *deleteDN;        
    LDAP        *ld;


    if (argc != 6)
    {
        printf("%s", usage);
        return(1);
    }

    ldapHost = argv[1];
    ldapPort = atoi(argv[2]);
    loginDN  = argv[3];
    password = argv[4];
    deleteDN = argv[5];

    /* Set LDAP version to 3 */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version);

    /* Initialize the LDAP session */
    if (( ld = ldap_init( ldapHost, ldapPort )) == NULL)
    {
        printf ( "\n\tLDAP session initialization failed\n");
        return( 1 );
    }
    printf ( "\n\tLDAP session initialized\n");

    /* Bind to the server */
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if (rc != LDAP_SUCCESS )
    {
        printf("ldap_simple_bind_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s ( ld );
        return( 1 );
    }
    printf("\n\tBind successful\n");


    /* Perform the delete operation. */
    rc = ldap_delete_ext_s( ld,         /* LDAP session handle */
                            deleteDN,   /* dn of the object to delete */
                            NULL,       /* server controls */
                            NULL );     /* client controls */

    if ( rc != LDAP_SUCCESS )
    {
        printf("\n\tldap_delete_ext_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s( ld );
        return(1);
    }
    printf("\n\tEntry deleted successfully: %s\n", deleteDN );

    ldap_unbind_s( ld );

    return( 0 );
}

