/* $Novell: /cm/ldap/src/cldap/samples/extensions/abortpo.c,v 1.8 2002/01/15 18:28:50 dsward Exp $ */
/*************************************************************************

   Copyright 1999, 2000, 2001 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   abortpo.c
***************************************************************************
   Description: The abortpo.c sample schedules a partition operation 
                and then aborts the pending operation on the specified 
                partition. Once eDirectory has started processing the 
                operation, it cannot be aborted.
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ldap.h>
#include <ldapx.h>

static char usage[] =
"\n Usage:   abortpo <host name> <port number> <login dn> <password>" 
"\n\t  <partition dn>\n"
"\n Example: abortpo acme.com 389 cn=admin,o=acme secret ou=sales,o=acme\n";

int main(int argc, char **argv)
{
    int     rc, version;           
    int     ldapPort;
    char    *ldapHost;
    char    *loginDN;
    char    *password;
    char    *partitionDN;
    char    *msg;
    LDAP    *ld;
      
    if (argc != 6)
    {
        printf("%s", usage);
        return (1);
    }
       
    ldapHost        =   argv[1];         
    ldapPort        =   atoi(argv[2]);
    loginDN         =   argv[3];
    password        =   argv[4];
    partitionDN     =   argv[5];

    
    /* Set LDAP version to v3 */   
    printf( "    Setting LDAP version to 3\n" );
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version );
   
    /* Initialize the LDAP session */
    printf( "    Initializing the LDAP session handle ...\n" );
    if ( (ld = ldap_init( ldapHost, ldapPort )) == NULL )
    {
        perror( "ldap_init" );
        return( 1 );  
    }
    
    /* Authenticate to the server */
    printf( "    Binding to the directory...\n" );
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if ( rc != LDAP_SUCCESS )
    {
        printf( "ldap_simple_bind_s: %s\n", ldap_err2string(rc));
        ldap_unbind_s( ld );
        return( 1 );
    }

    /* 
     * Split a partition to give us a pending operation we can abort. 
     * Once DS starts the actual operation, it cannot be aborted.
     */
    printf( "    Calling ldap_split_partition...\n" );
    rc = ldap_split_partition( 
                ld,                         /* LDAP session handle */
                partitionDN,                /* dn of the partition */
                LDAP_ENSURE_SERVERS_UP );   
    printf("\n    ldap_split_partition is pending...");

    /*
     *  Novell extended operation to abort a pending eDirectory operation 
     */
    printf("\n\n    Attempting to abort the operation...\n");
    rc = ldap_abort_partition_operation( 
                ld,                 /* LDAP session handle */ 
                partitionDN,        /* dn of the partition */
                0 );                /* reserved */

    if ( rc != LDAP_SUCCESS )
    {           
        /* print out LDAP server message */         
        if(ldap_get_option (ld, LDAP_OPT_ERROR_STRING, &msg ) == 
                                                      LDAP_OPT_SUCCESS ) {
            
            if ( (msg != NULL) && (strlen(msg) != 0) )
                printf("\n    LDAP server message: %s", msg);
        }
        /* print out LDAP error message */
        fprintf( stderr, "    LDAP error  message: ldap_abort_partition_" 
                           "operation: %s (%d)\n", ldap_err2string( rc ), rc);
        
        printf("\n    ldap_abort_partition_operation failed\n\n");
        ldap_unbind_s( ld );
        return( 1 );
    }
            
    printf("\n    ldap_abort_partiton_operation succeeded\n\n");
  
    ldap_unbind_s( ld );
    return 0;
}

