/* $Novell: /cm/ldap/src/cldap/samples/extensions/chgrepl.c,v 1.10 2002/01/15 18:28:50 dsward Exp $ */
/*************************************************************************

   Copyright 1999, 2000, 2001 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   chrepl.c 
***************************************************************************
   Description: The chgrepl.c sample changes the type of the specified 
                replica on the specified server. You cannot directly
                change the type of a master replica. eDirectory 
                automatically changes the master replica's type when a 
                subordinate replica is changed to a master replica.

***************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ldap.h>
#include <ldapx.h>

static char usage[] =
"\n Usage:   chgrepl <host name> <port number> <login dn> <password>" 
"\n          <replica dn> <new replica type> <server entry dn>\n"
"\n Example: chgrepl acme.com 389 cn=admin,o=acme secret"
"\n          ou=sales,o=acme 1 cn=myServer,o=acme\n"
"\n          <new replica type>: 0-master, 1-read/write, 2-read only\n";

int main(int argc, char **argv)
{
     
    int      rc, version; 
    int      ldapPort;
    char     *ldapHost;       
    char     *loginDN; 
    char     *password; 
    char     *replicaDN; 
    int      newReplicaType; 
    char     *serverName;
    char     *msg;
    LDAP     *ld;

    if (argc != 8)
    {
        printf("%s", usage);
        return (1);
    }
    
    ldapHost        =  argv[1];
    ldapPort        =  atoi(argv[2]);
    loginDN         =  argv[3];
    password        =  argv[4];
    replicaDN       =  argv[5];
    newReplicaType  =  atoi(argv[6]);  /* 0-master, 1-read/write, 2-read only */
    serverName      =  argv[7];

      
    /* Set LDAP version to v3 */   
    printf( "    Setting LDAP version to 3\n" );
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version );
   
    /* Initialize the LDAP session */
    printf( "    Initializing the LDAP session handle ...\n" );
    if ( (ld = ldap_init( ldapHost, ldapPort )) == NULL )
    {
        perror( "ldap_init" );
        return( 1 );  
    }
    
    /* Authenticate to the server */
    printf( "    Binding to the directory...\n" );
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if ( rc != LDAP_SUCCESS )
    {
        printf( "ldap_simple_bind_s: %s\n", ldap_err2string(rc));
        ldap_unbind_s( ld );
        return( 1 );
    }

    /* 
     *  Novell extended operation to change a replica type. 
     */
    printf( "    Calling ldap_change_replica_type...\n" );
    rc = ldap_change_replica_type(   
                ld,                     /* LDAP session handle */
                replicaDN,              /* dn of the replica */
                serverName,             /* server entry dn */                          
                (LDAP_REPLICA_TYPE)newReplicaType,
                LDAP_ENSURE_SERVERS_UP );           

    if ( rc != LDAP_SUCCESS )
    {
        /*  print out LDAP server message */
        if(ldap_get_option (ld, LDAP_OPT_ERROR_STRING, &msg ) == 
                                                      LDAP_OPT_SUCCESS ) {
            
            if ( (msg != NULL) && (strlen(msg) != 0) )
                printf("\n    LDAP server message: %s", msg);
        }
        /* print out LDAP error message */
        fprintf( stderr, "    LDAP error  message: ldap_change_replica_type:" 
                            " %s (%d)\n", ldap_err2string( rc ), rc);        
        
        printf("\n    ldap_change_replica_type failed\n\n");
        ldap_unbind_s( ld );
        return( 1 );
    }
             
    printf("\t\tldap_change_replica_type succeeded\n\n");

    ldap_unbind_s( ld );
    return 0;
}
