/* $Novell: /cm/ldap/src/cldap/samples/extensions/getbinddn.c,v 1.2 2002/01/15 18:28:50 dsward Exp $ */
/*************************************************************************

   Copyright 2001 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   getbinddn.c 
***************************************************************************
   Description: The getbinddn.c sample returns the distinguished name of 
                the client associated with the LDAP session handle, or 
				the "bind dn".
***************************************************************************/ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ldap.h>
#include <ldapx.h>

static char usage[] =
"\n Usage:   getbinddn <host name> <port number> <login dn> <password>\n" 
"\n Example: getbinddn acme.com 389 cn=admin,o=acme secret\n";

int main(int argc, char **argv)
{
    
    int     rc, version, ldapPort;              
    char    *ldapHost;        
    char    *loginDN;  
    char    *password; 
    char    *bindDN = NULL;
    char    *msg;
    LDAP    *ld;
    
    if (argc != 5)
    {
        printf("%s", usage);
        return (1);
    }
                     
    ldapHost    =   argv[1];         
    ldapPort    =   atoi(argv[2]);
    loginDN     =   argv[3];
    password    =   argv[4];
        

    /* Set LDAP version to v3 */   
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version );
   
    /* Initialize the LDAP session */
    if (( ld = ldap_init( ldapHost, ldapPort )) == NULL) {
        printf ( "\n    LDAP session initialization failed\n");
        return( 1 );
    }
    printf ( "\n    LDAP session initialized\n");

    /* Bind to the server */
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if (rc != LDAP_SUCCESS ) {
        printf("ldap_simple_bind_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s ( ld );
        return( 1 );
    }
    printf("\n    Bind successful\n");

    /* 
     * Novell extended operation to return the dn of the entry we
     * are logged in as.
     */
    printf( "    Calling ldap_get_bind_dn...\n" );
    rc = ldap_get_bind_dn( ld,         /* LDAP session handle */ 
                           &bindDN);   /* returned identity name */

    if ( rc != LDAP_SUCCESS )
    {
        /* print out LDAP server message */         
        if(ldap_get_option(ld, LDAP_OPT_ERROR_STRING, &msg ) == 
                                                      LDAP_OPT_SUCCESS ) {
            
            if ( (msg != NULL) && (strlen(msg) != 0) )
                printf("\n    LDAP server message: %s", msg);                
        }
        /* print out LDAP error message */
        fprintf( stderr, "    LDAP error  message: ldap_get_bind_dn:"
                                " %s (%d)\n", ldap_err2string( rc ), rc);
        
        printf("\n    ldap_get_bind_dn failed\n\n");
        ldap_unbind_s( ld );
        return( 1 );
    }
            
    printf("\n    ldap_get_bind_dn succeeded\n");
    printf("\n    bind dn: %s\n\n", bindDN);
        
    ldapx_memfree( bindDN );
    ldap_unbind_s( ld );
    return 0;
}
