/* $Novell: /cm/ldap/src/cldap/samples/extensions/getpriv.c,v 1.9 2002/01/15 18:28:49 dsward Exp $ */
/*************************************************************************

   Copyright 1999-2001 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   getpriv.c 
***************************************************************************
   Description: The getpriv.c sample returns the effective rights of 
                the specified entry (trustee) to the specified attribute 
                of the specified subject (object).
***************************************************************************/ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ldap.h>
#include <ldapx.h>

static char usage[] =
"\n Usage:   getpriv <host name> <port number> <login dn> <password>" 
"\n\t  <trustee dn> <object dn> <attribute name>\n"
"\n Example: getpriv acme.com 389 cn=admin,o=acme secret"
"\n\t  cn=admin,o=acme cn=James,ou=sales,o=acme cn\n";

int main(int argc, char **argv)
{
    int   rc, version, privileges, ldapPort;        
    char  *ldapHost;        
    char  *loginDN; 
    char  *password; 
    char  *objectDN;
    char  *trusteeDN;
    char  *attrName;
    char  *msg;
    LDAP  *ld;

    if (argc != 8)
    {
        printf("%s", usage);
        return(1);
    }


    ldapHost   =  argv[1];         
    ldapPort   =  atoi(argv[2]);
    loginDN    =  argv[3];
    password   =  argv[4];
    objectDN   =  argv[5];
    trusteeDN  =  argv[6];
    attrName   =  argv[7];   

    /* Set LDAP version to v3 */   
    printf( "    Setting LDAP version to 3.0\n" );
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version );

    /* Initialize the LDAP session */
    printf( "    Getting the handle to the LDAP connection...\n" );
    if ( (ld = ldap_init( ldapHost, ldapPort )) == NULL )
    {
        perror( "ldap_init" );
        return( 1 );   
    }

    /* Authenticate to the server */
    printf( "    Binding to the directory...\n" );
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if ( rc != LDAP_SUCCESS )
    {
        printf( "ldap_simple_bind_s: %s\n", ldap_err2string(rc));
        ldap_unbind_s( ld );
        return( 1 );
    }


    /* 
     * Novell extended operation to get the effective privileges 
     * of a trustee upon an entry's attribute.
     */
    printf( "    Calling ldap_get_effective_privileges...\n" );
    rc = ldap_get_effective_privileges(
                                      ld,            /* LDAP session handle */ 
                                      objectDN,      /* object dn */
                                      trusteeDN,     /* trustee dn */
                                      attrName,      /* attribute name */
                                      &privileges);  /* privileges */

    if ( rc != LDAP_SUCCESS )
    {
       /* print out LDAP server message */         
        if(ldap_get_option(ld, LDAP_OPT_ERROR_STRING, &msg ) == 
                                                      LDAP_OPT_SUCCESS ) {
            
            if ( (msg != NULL) && (strlen(msg) != 0) )
                printf("\n    LDAP server message: %s", msg);                
        }
        /* print out LDAP error message */
        fprintf( stderr, "    LDAP error  message: ldap_get_effective_" 
                   "privileges: %s (%d)\n", ldap_err2string( rc ), rc);
        
        printf("\n    ldap_get_effective_privileges failed\n\n");
        ldap_unbind_s( ld );
        return( 1 );
    }
            
    printf("\n    ldap_get_effective_privileges succeeded\n\n");

    printf("\tTrustee:   %s\n", trusteeDN);
    printf("\tObject:    %s\n", objectDN);
    printf("\tAttribute: %s\n", attrName);

    if ( privileges & LDAP_DS_ATTR_COMPARE )
        printf("\n\tTrustee has compare rights to the attribute.\n\n");
    else
        printf("\n\tTrustee has NO compare rights to the attribute.\n\n");

    if ( privileges & LDAP_DS_ATTR_READ )
        printf("\n\tTrustee has read rights to the attribute.\n\n"); 
    else
        printf("\n\tTrustee has NO read rights to the attribute.\n\n"); 

    if ( privileges & LDAP_DS_ATTR_WRITE )
        printf("\n\tTrustee has write rights to the attribute.\n\n"); 
    else
        printf("\n\tTrustee has NO write rights to the attribute.\n\n"); 
    
    if ( privileges & LDAP_DS_ATTR_SELF )
        printf("\n\tTrustee can add or delete its name as a value "
               "of the attribute.\n\n");
    else
        printf("\n\tTrustee CANNOT add or delete its name as a value "
               "of the attribute.\n\n");

    if ( privileges & LDAP_DS_ATTR_SUPERVISOR )
        printf("\n\tTrustee has all rights to the the object's attributes\n\n");
    else
        printf("\n\tTrustee does not have all rights to the the object's attributes\n\n");

    if ( privileges & LDAP_DS_ATTR_INHERIT_CTL )
        printf("\n\tTrustee inherits the rights granted in the ACL.\n\n");
    else
        printf("\n\tTrustee does not inherit the rights granted in the ACL.\n\n");


    ldap_unbind_s( ld );
    return 0;
}

