/* $Novell: /cm/ldap/src/cldap/samples/extensions/getrinfo.c,v 1.7 2002/01/15 18:28:49 dsward Exp $ */
/*************************************************************************

   Copyright 1999, 2000 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   getrinfo.c 
***************************************************************************
   Description: The getrinfo.c returns information about a replica. 
                The replica is identified by specifying the distinguished 
                name of the replica's partition and the server it is on.
***************************************************************************/ 



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ldap.h>
#include <ldapx.h>

static char usage[] =
"\n Usage:   getrinfo <host name> <port number> <login dn> <password> " 
"\n\t  <partition dn> <serverDN>\n"
"\n Example: getrinfo acme.com 389 cn=admin,o=acme secret ou=sales,o=acme"
"\n\t  cn=myServer,o=acme\n";

int main(int argc, char **argv)
{      
    int   rc, version, ldapPort;
    char  *ldapHost;             
    char  *loginDN;
    char  *password;
    char  *partitionDN;
    char  *serverDN;
    char  *msg;
    LDAPReplicaInfo replicaInfo;
    LDAP  *ld;

    if ( argc != 7 )
    {
        printf("%s", usage);
        return (1);
    }
       
    ldapHost    =  argv[1];         
    ldapPort    =  atoi(argv[2]);   
    loginDN     =  argv[3];
    password    =  argv[4];
    partitionDN =  argv[5];
    serverDN    =  argv[6];
          
    /* Set LDAP version to v3 */   
    printf( "    Setting LDAP version to 3.0\n" );
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version );

    /* Initialize the LDAP session */
    printf( "    Getting the handle to the LDAP connection...\n" );
    if ( (ld = ldap_init( ldapHost, ldapPort )) == NULL ) 
    {
        perror( "ldap_init" );
        return ( 1 );   
    }      

    /* Authenticate to the server */ 
    printf( "    Binding to the directory...\n" );
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if ( rc != LDAP_SUCCESS ) 
    {
        printf( "ldap_simple_bind_s: %s\n", ldap_err2string(rc));
        ldap_unbind_s( ld );
        return( 1 );
    }

    /* 
     * Novell extended operation to get replica information.
     */
    printf( "    Calling ldap_get_replica_info...\n" );
    rc = ldap_get_replica_info(
                        ld,            /* LDAP session handle */
                        partitionDN,   /* Partition dn */
                        serverDN,      /* server dn */
                        &replicaInfo); /* struct to receive replica info */

    if ( rc != LDAP_SUCCESS ) 
    {
        /* print out LDAP server message */         
        if(ldap_get_option (ld, LDAP_OPT_ERROR_STRING, &msg ) == 
                                                      LDAP_OPT_SUCCESS ) {
            
            if ( (msg != NULL) ) {
                printf("\n    LDAP server message: ");
                if ( strlen(msg) != 0 )
                    printf("%s\n", msg);
                else
                    printf("none\n");
                ldap_memfree( msg );
            }
        }
        /* print out LDAP error message */
        fprintf( stderr, "    LDAP error  message: ldap_get_replica_info:" 
                            " %s (%d)\n", ldap_err2string( rc ), rc);
        
        printf("\n    ldap_get_replica_info failed\n\n");
        ldap_unbind_s( ld );
        return( 1 );
    }

    printf("\n    ldap_get_replica_info succeeded\n\n");

    printf("\n    Replica info: ");

    printf("\n        Partition root ID: %d", replicaInfo.rootID);
    printf("\n        Replica state: %d", replicaInfo.state);
    printf("\n        tModification time: %d", replicaInfo.modificationTime);
    printf("\n        tPurge time: %d", replicaInfo.purgeTime);
    printf("\n        Local replica ID: %d", replicaInfo.localReplicaID);
    printf("\n        Replica type: %d", replicaInfo.replicaType);
    printf("\n        partition DN: %s", replicaInfo.namingContextDN);
    printf("\n        Flag: %d", replicaInfo.flags);

    printf("\n\n");

    ldap_unbind_s( ld );
    return 0;
}

