/* $Novell: /cm/ldap/src/cldap/samples/extensions/listrepl.c,v 1.7 2002/01/15 18:28:48 dsward Exp $ */
/*************************************************************************

   Copyright 1999-2001 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
    listrepl.c
***************************************************************************
   Description: The listrepl.c sample lists all the replicas on the 
                specified LDAP server.
***************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ldap.h>
#include <ldapx.h>

static char usage[] =
"\n Usage:   listrepl <host name> <port number> <login dn> <password>" 
"\n\t  <serverDN>\n"
"\n Example: listrepl acme.com 389 cn=admin,o=acme secret cn=myServer,o=acme\n";


int main(int argc, char **argv)
{
    int   i, rc, version;
    char  *ldapHost;         
    int   ldapPort;   
    char  *loginDN;
    char  *password;     
    char  *serverDN;
    char  *msg;
    char  **replicaList = NULL;
    LDAP  *ld;

    if ( argc != 6 )
    {
        printf("%s", usage);
        return(1);
    }

    ldapHost      =  argv[1];         
    ldapPort      =  atoi(argv[2]);   
    loginDN       =  argv[3];
    password      =  argv[4];     
    serverDN      =  argv[5];   

    /* Set LDAP version to v3 */   
    printf( "    Setting LDAP version to 3\n" );
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version );

    /* Initialize the LDAP session */
    printf( "    Initializing the LDAP session handle ...\n" );
    if ( (ld = ldap_init( ldapHost, ldapPort )) == NULL )
    {
        perror( "ldap_init" );
        return( 1 );  
    }

    /* Authenticate to the server */
    printf( "    Binding to the directory...\n" );
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if ( rc != LDAP_SUCCESS )
    {
        printf( "ldap_simple_bind_s: %s\n", ldap_err2string(rc));
        ldap_unbind_s( ld );
        return( 1 );
    }

    /* 
     * Novell extended operation to list the replicas on this server.
     */
    printf( "    Calling ldap_list_replicas...\n" );
    rc = ldap_list_replicas(
                       ld,            /* LDAP session handle */
                       serverDN,      /* dn of the server */
                       &replicaList); /* char array to receive replica list */

    if ( rc != LDAP_SUCCESS )
    {
        /* print out LDAP server message */         
        if(ldap_get_option (ld, LDAP_OPT_ERROR_STRING, &msg ) == 
                                                      LDAP_OPT_SUCCESS ) {
            
            if ( (msg != NULL) && (strlen(msg) != 0) )
                printf("\n    LDAP server message: %s", msg);
        }
        /* print out LDAP error message */
        fprintf( stderr, "    LDAP error  message: ldap_list_replicas:" 
                            " %s (%d)\n", ldap_err2string( rc ), rc);
        printf("\n    ldap_list_replicas failed\n\n");
        ldap_unbind_s( ld );
        return( 1 );
    }

    printf("\n    ldap_list_replicas succeeded\n\n");

    if(replicaList)
    {
        printf("\n    replicas existing in %s: ", serverDN);
        for ( i = 0; replicaList[i] != NULL; i++)
            printf("        %s\n", replicaList[i] );
    }

    printf("\n\n");

    ldap_value_free( replicaList );
    
    ldap_unbind_s( ld );
    
    return 0;
}

