/* $Novell: /cm/ldap/src/cldap/samples/extensions/schsync.c,v 1.7 2002/01/15 18:28:46 dsward Exp $ */
/*************************************************************************

   Copyright 1999-2001 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   schsync.c 
***************************************************************************
   Description: The schsync.c sample schedules schema synchronization after 
                the specified delay.
 
***************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ldap.h>
#include <ldapx.h>

static char usage[] =
"\n Usage:   schsync <LDAP server> <port number> <login dn> <password>" 
"\n\t  <partition root> <delay seconds>\n"
"\n Example: schsync acme.com 389 cn=admin,o=acme secret o=acme 1\n";

int main(int argc, char **argv)
{

    int   rc, version, ldapPort, delay;               
    char  *ldapHost;         
    char  *loginDN;
    char  *password;
    char  *serverName;
    char  *msg;
    LDAP  *ld;

    if (argc != 7)
    {
        printf("%s", usage);
        return (1);
    }                     
    
    ldapHost    =  argv[1];         
    ldapPort    =  atoi(argv[2]);
    loginDN     =  argv[3];
    password    =  argv[4];
    serverName  =  argv[5];
    delay       =  atoi(argv[6]);


    /* Set LDAP version to v3 */   
    printf( "Setting LDAP version to 3\n" );
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version );
   
    /* Initialize the LDAP session */
    printf( "Initializing the LDAP session handle ...\n" );
    if ( (ld = ldap_init( ldapHost, ldapPort )) == NULL )
    {
        perror( "ldap_init" );
        return( 1 );  
    }
    
    /* Authenticate to the server */
    printf( "Binding to the directory...\n" );
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if ( rc != LDAP_SUCCESS )
    {
        printf( "ldap_simple_bind_s: %s\n", ldap_err2string(rc));
        ldap_unbind_s( ld );
        return( 1 );
    }

    /* 
     * Novell extended operation to synchronize the schema.
     */
    printf( "Calling ldap_request_schema_sync...\n" );
    rc = ldap_request_schema_sync(
                            ld,                 /* LDAP session handle */
                            serverName,         /* server name */
                            delay);             /* delay time */

    if ( rc != LDAP_SUCCESS )
    {
        /* print out LDAP server message */         
        if(ldap_get_option (ld, LDAP_OPT_ERROR_STRING, &msg ) == 
                                                      LDAP_OPT_SUCCESS ) {
            if ( (msg != NULL) && (strlen(msg) != 0) )
                printf("\n    LDAP server message: %s", msg);
        }
        /* print out LDAP error message */
        fprintf( stderr, "    LDAP error  message: ldap_request_schema_sync" 
                                  ": %s (%d)\n", ldap_err2string( rc ), rc);
        
        
        fprintf( stderr, "ldap_request_schema_sync: %s\n", 
                               ldap_err2string( rc ));
        printf("\t\tSchema synchronization failed\n");
        ldap_unbind_s( ld );
        return( 1 );
    }
       
    printf("\t\tSchema synchronization scheduled\n");

    ldap_unbind_s( ld );   
    return 0;
}
