/* $Novell: /ldap/src/cldap/samples/modattrs.c,v 1.4 2000/08/21 22:59:05 vtag Exp $ */
/*************************************************************************

   Copyright 1999, 2000 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   modattrs.c 
***************************************************************************
   Description: The modattrs.c sample shows how to modify the attributes 
                of the entry created with the addentry.c sample. 
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ldap.h>

static char usage[] =
"\n Usage:   modattrs <host name> <port number> <login dn> <password>"
"\n\t <modify dn> \n" 
"\n Example: modattrs acme.com 389 cn=admin,o=acme secret"
"\n\t cn=james,ou=sales,o=acme\n";

int main( int argc, char **argv)
{ 

    int         version, ldapPort, rc; 
    char        *ldapHost, *loginDN, *password, *modifyDN;
    char        *mailValues[2], *phoneValues1[2], *phoneValues2[2] ;
    LDAP        *ld;
    LDAPMod     modMail, modPhone1, modPhone2, *modify[4];

    if (argc != 6)
    {
        printf("%s", usage);
        return(1);
    }

    ldapHost    =   argv[1];
    ldapPort    =   atoi(argv[2]);
    loginDN     =   argv[3];
    password    =   argv[4];
    modifyDN    =   argv[5];

    /* Set LDAP version to 3 */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version);

    /* Initialize the LDAP session */
    if (( ld = ldap_init( ldapHost, ldapPort )) == NULL)
    {
        printf ( "\n\tLDAP session initialization failed\n");
        return( 1 );
    }
    printf ( "\n\tLDAP session initialized\n");

    /* Bind to the server */
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if (rc != LDAP_SUCCESS )
    {
        printf("ldap_simple_bind_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s ( ld );
        return( 1 );
    }
    printf("\n\tBind successful\n");


    /* 
     * To modify the attributes of an entry
     *    1  Specify the modify actions    
     *    2. Specify the attribute name to be modified 
     *    3. Specify the value of the attribute
     *    4. Add to an array of LDAPMod structures
     *    5. Call ldap_modify_ext_s
     */


    /* 
     * LDAP_MOD_REPLACE succeeds whether the value already exists or not.
     */
    modMail.mod_op = LDAP_MOD_REPLACE;
    modMail.mod_type = "mail";
    mailValues[0] = "jimmy@acme.com";
    mailValues[1] = NULL;
    modMail.mod_values = mailValues;

    /* 
     * LDAP_MOD_ADD gives an error if the value already exists.
     */
    modPhone1.mod_op = LDAP_MOD_ADD;
    modPhone1.mod_type = "telephoneNumber";
    phoneValues1[0] = "1 234 567 8910";
    phoneValues1[1] = NULL;
    modPhone1.mod_values = phoneValues1;

    /* 
     * LDAP_MOD_DELETE will give an error if the value does not exist.
     * In this example, we delete the telephone number we just added.
     */
    modPhone2.mod_op = LDAP_MOD_DELETE;
    modPhone2.mod_type = "telephoneNumber";
    phoneValues2[0] = "1 234 567 8910";
    phoneValues2[1] = NULL;
    modPhone2.mod_values = phoneValues2;

    modify[0] = &modMail;
    modify[1] = &modPhone1;
    modify[2] = &modPhone2;
    modify[3] = NULL;

    /* Modify the attributes */
    rc= ldap_modify_ext_s( ld,         /* LDAP session handle */
                           modifyDN,   /* the object to modify */
                           modify,     /* array of LDAPMod structures */
                           NULL,       /* server controls */
                           NULL);      /* client controls */

    if ( rc != LDAP_SUCCESS )
    {
        printf("ldap_modify_ext_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s( ld );
        return(1);
    }
    printf("\n\t%s modified successfully.\n", modifyDN );

    ldap_unbind_s( ld );

    return( 0 );
}

