/* $Novell: /cm/ldap/src/cldap/samples/modifyACL.c,v 1.5 2001/12/17 22:20:01 dsteck Exp $ */
/******************************************************************************

   Copyright 2001 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

*******************************************************************************
   modifyACL.c 
*******************************************************************************
   Description: modifyACL.c first modifies entryDN's ACL values to grant 
                trusteeDN the read, write, and delete entry rights. It then 
                dispalys entryDN's ACL values. Finaly it removes entryDN's 
                modified ACL value.
                
                ACL (Access Control List) is a multivalued optional attribute.
                An entry's ACL values define which other entries (trustees) 
                have what kinds of access to the entry itself ([Entry Rights])
                and its attributes ([All Attributes Rights] or single attribute
                rights). 
                
                Each of ACL values is in the format of
                    "privileges#scope#subjectname#protectedattrname".
                privileges:        ORed bits to indicate the rights.
                scope:             either 'entry' or 'subtree'.
                subjectname:       trustee DN.
                protectedattrname: [Entry Rights], or [All Attributes Rights],
                                   or a single attribute name.
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ldap.h>

int  findACLValues( LDAP* , char *);
void printACLValue( char* );

/* eDirectory [All Attribute Rights] flags */
#define LDAP_DS_ATTR_COMPARE        0x00000001L
#define LDAP_DS_ATTR_READ           0x00000002L
#define LDAP_DS_ATTR_WRITE          0x00000004L
#define LDAP_DS_ATTR_SELF           0x00000008L
#define LDAP_DS_ATTR_SUPERVISOR     0x00000020L

/* eDirectory [Entry Rights] flags */
#define LDAP_DS_ENTRY_BROWSE        0x00000001L
#define LDAP_DS_ENTRY_ADD           0x00000002L
#define LDAP_DS_ENTRY_DELETE        0x00000004L
#define LDAP_DS_ENTRY_RENAME        0x00000008L
#define LDAP_DS_ENTRY_SUPERVISOR    0x00000010L

static char usage[] =
"\n Usage:   modifyACL <host name> <port number> <login dn> <password>"
"\n                    <entry dn> <trustee dn>"
"\n Example: modifyACL Acme.com 389 cn=Admin,o=Acme secret"
"\n                    cn=test,ou=Sales,o=Acme cn=trustee,o=Acme\n";

int main( int argc, char **argv)
{ 
    int      version, ldapPort, rc, privileges; 
    char     *ldapHost, *loginDN, *password, *entryDN, *trusteeDN;
    char     *modValues[2], aclValue[128];
    LDAPMod  modACL, *modify[2];
    LDAP     *ld;

    if (argc != 7) {
        printf("%s", usage);
        return(1);
    }

    ldapHost   = argv[1];
    ldapPort   = atoi(argv[2]);
    loginDN    = argv[3];
    password   = argv[4];
    entryDN    = argv[5];
    trusteeDN  = argv[6]; 

    /* Set LDAP version to 3 */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version);

    /* Initialize the LDAP session */
    if (( ld = ldap_init( ldapHost, ldapPort )) == NULL) {
        printf ( "\n    LDAP session initialization failed\n");
        return( 1 );
    }
    printf ( "\n    LDAP session initialized\n");

    /* Bind to the server */
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if (rc != LDAP_SUCCESS ) {
        printf("\n    ldap_simple_bind_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s ( ld );
        return( 1 );
    }
    printf("\n    Bind successful\n");

    /* Encode ACL value */ 
    privileges = LDAP_DS_ENTRY_BROWSE;
    privileges |= LDAP_DS_ENTRY_ADD;
    privileges |= LDAP_DS_ENTRY_DELETE;
    sprintf(aclValue,"%d", privileges);
    strcat(aclValue, "#entry#");
    strcat(aclValue, trusteeDN);
    strcat(aclValue, "#[Entry Rights]");

    /* Populate LDAPMod structures with the values */
    modACL.mod_op     = LDAP_MOD_ADD;
    modACL.mod_type   = "acl";
    modValues[0]      = aclValue;
    modValues[1]      = NULL;
    modACL.mod_values = modValues;
    modify[0]         = &modACL;   
    modify[1]         = NULL;

    /* Modify entryDN's ACL value */ 
    printf( "\n    Entry DN: %s", entryDN );
    printf( "\n    Trustee DN: %s\n", trusteeDN );
    printf( "\n    Modifying ACL values ..." );
    rc = ldap_modify_ext_s( ld,         /* LDAP session handle */
                            entryDN,    /* the object to modify */
                            modify,     /* array of LDAPMod structures */
                            NULL,       /* server controls */
                            NULL);      /* client controls */

    if ( rc != LDAP_SUCCESS ) {
        printf("\n    ldap_modify_ext_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s( ld );
        return(1);
    }
    
    printf("\n    Modified  ACL values to grant the trustee the 'read', "
           "\n    'write', and 'delete' entry rights.\n");

    /* List entryDN's ACL attribute values */
    rc = findACLValues( ld, entryDN);

    if ( rc != LDAP_SUCCESS ) {
        printf("\n    Failed to read the entry: %s\n", ldap_err2string( rc ));
        ldap_unbind_s( ld );
        return(1);
    }
    
    /* Remove modified entryDN's ACL value */
    printf( "\n\n    Removing modified ACL value ...\n" );
    modify[0]->mod_op = LDAP_MOD_DELETE;
    rc= ldap_modify_ext_s( ld,         /* LDAP session handle */
                           entryDN,    /* the object to modify */
                           modify,     /* array of LDAPMod structures */
                           NULL,       /* server controls */
                           NULL);      /* client controls */
    if ( rc != LDAP_SUCCESS ) {
        printf("\n    ldap_modify_ext_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s( ld );
        return(1);
    }
    printf( "    Removed modified ACL value.\n" );

    ldap_unbind_s( ld );
    return( 0 );
}

/* 
 * listACLValues() searches the directory for the entry to 
 * get the entry's acl attribute 
 */
int findACLValues( LDAP *ld, char *entryDN) 
{    
    int         rc, i;
    char        **values, *attribute, *attrs[] = { "acl", NULL };
    struct      timeval timeOut;    
    BerElement  *ber;
    LDAPMessage *searchResult, *entry;

    timeOut.tv_sec    = 10L;
    timeOut.tv_usec   = 0L;
            
    /* Search the directory to get the entry's ACL attribute */
    rc = ldap_search_ext_s(  
        ld,                    /* LDAP session handle */
        entryDN,               /* object to read */
        LDAP_SCOPE_BASE,       /* search scope */
        "(objectclass=*)",     /* search filter */
        attrs,                 /* return acl attribute only */
        0,                     /* return both the ACL and its values */
        NULL,                  /* server controls */
        NULL,                  /* client controls */
        &timeOut,              /* time out */
        LDAP_NO_LIMIT,         /* no size limit */
        &searchResult );       /* returned results */

    if ( rc != LDAP_SUCCESS )        
        return rc;                                   

    /* Search returns entryDN's ACL, print its values out */
    entry   =   ldap_first_entry( ld, searchResult );
    attribute = ldap_first_attribute( ld, entry, &ber );               
        
    printf("\n    ===========================================================");
    printf("\n    ACL values of after modification: " );
    printf("\n    ===========================================================");
                
    if (( values = ldap_get_values( ld, entry, attribute)) != NULL )
        for ( i = 0; values[ i ] != NULL; i++ )
            printACLValue( values[i] );                  
           
    ldap_value_free( values );
    ldap_memfree( attribute );
    ber_free(ber, 0);
    ldap_msgfree( searchResult );

    return rc;
}

/*
 * printACLValue() takes aclValue as input. It parses
 * and prints the ACL value.
 */
void printACLValue( char* aclValue ) 
{        
    int  i, privileges;
    char privString[128], *pdest, scope[16], trusteeDN[128];
    char *protectedAttrName, privs[128];    
                           
    /* Change each '#' into ' ' */
    for ( i = 0; i < 3; i++ ) {
        pdest = strchr( aclValue, '#');        
        aclValue[pdest-aclValue] = ' ';
    }
    
    sscanf(aclValue, "%d", &privileges );
    sscanf(aclValue, "%s", privString);
    i =  strlen( privString );
    sscanf(aclValue + i + 1, "%s", scope );
    i += strlen( scope );
    sscanf(aclValue + i + 2, "%s", trusteeDN);
    i += strlen( trusteeDN );
    /* Protected attribute name may contain space, use 
     * pointer assignment instead of sscanf() 
     */
    protectedAttrName = aclValue + i + 3;

    if (strcmp( protectedAttrName, "[Entry Rights]") == 0) {
        /* Decode the [Entry Rights]   */
        strcpy(privs,"");
        if ( privileges & LDAP_DS_ENTRY_BROWSE )
            strcat(privs, "BrowseEntry ");
        if ( privileges & LDAP_DS_ENTRY_ADD )
            strcat(privs, "AddEntry ");
        if ( privileges & LDAP_DS_ENTRY_DELETE )
            strcat(privs, "DeleteEntry ");
        if ( privileges & LDAP_DS_ENTRY_RENAME )
            strcat(privs, "RenameEntry ");
        if ( privileges & LDAP_DS_ENTRY_SUPERVISOR )
            strcat(privs, "Supervisor");            
    }
    else {
        /* Decode [All Attributes Rights] or attribute rights */
        strcpy(privs,"");
        if ( privileges & LDAP_DS_ATTR_COMPARE )
            strcat(privs, "CompareAttributes ");        
        if ( privileges & LDAP_DS_ATTR_READ )
            strcat(privs, "ReadAttributes ");        
        if ( privileges & LDAP_DS_ATTR_WRITE )
            strcat(privs, "Write/Add/DeleteAttributes ");
        if ( privileges & LDAP_DS_ATTR_SELF )
            strcat(privs, "Add/DeleteSelf ");
        if ( privileges & LDAP_DS_ATTR_SUPERVISOR )
            strcat(privs, "Supervisor");
    }    
             
    /* Print out each ACL value */
    printf("\n    Trustee DN: %s", trusteeDN);
    printf("\n    Scope: %s", scope);
    printf("\n    Protected attribute name: %s", protectedAttrName);
    printf("\n    Privileges: %s", &privs);
    printf("\n    ------------------------------------------------------");
}
