/* $Novell: /cm/ldap/src/cldap/samples/modpass.c,v 1.4 2001/12/17 22:20:00 dsteck Exp $ */
/*************************************************************************

   Copyright 1999, 2000 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   modpass.c     (For users - does not require admin privilege)
***************************************************************************
   
   Description: The modpass.c sample shows how to modify your own 
                password, giving both the old and new password.
                
                Unless the caller has admin privilegs, eDirectory requires both
                the old and new passwords in order to change a password.
                
                NOTE:  This method requires the Novell Server to be running
                eDirectory version 8.3x or later.  See the setpass.c sample 
                which works on any Novell LDAP server but requires that the 
                caller have admin privilege.

                Two LDAPMod structures are created.
                - The first deletes the value of the userPassword attribute with
                  an LDAP_MOD_DELETE operation, specifying the old password.
                - The second one adds the new password to the userPassword
                  attribute using an LDAP_MOD_ADD operation.
                  
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ldap.h>

static char usage[] =
"\n Usage:   modpass <host name> <port number> <login dn> <old password>"
"\n          <new password>\n" 
"\n Example: modpass Acme.com 389 cn=james,ou=Sales,o=Acme oldpass"
"\n          newpass\n";

int main( int argc, char **argv)
{ 

    LDAP        *ld;
    int         version, ldapPort, rc; 
    char        *ldapHost, *loginDN, *password, *newPassword;
    char        *oldValues[2], *newValues[2];
    LDAPMod     delmod, addmod, *modify[3];

    if (argc != 6)
    {
        printf("%s", usage);
        return(1);
    }

    ldapHost    =   argv[1];
    ldapPort    =   atoi(argv[2]);
    loginDN     =   argv[3];
    password    =   argv[4];
    newPassword =   argv[5];

    /* Set LDAP version to 3 */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version);

    /* Initialize the LDAP session */
    if (( ld = ldap_init( ldapHost, ldapPort )) == NULL)
    {
        printf ( "\n\tLDAP session initialization failed\n");
        return( 1 );
    }
    printf ( "\n\tLDAP session initialized\n");

    /* Bind to the server */
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if (rc != LDAP_SUCCESS )
    {
        printf("ldap_simple_bind_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s ( ld );
        return( 1 );
    }
    printf("\n\tBind successful\n");


    /* 
     * Delete the old password, add the new password.
     * eDirectory requires both passwords unless caller has admin privilege.
     */
    delmod.mod_op   = LDAP_MOD_DELETE;
    delmod.mod_type = "userPassword";
    /* "Invalid credentials" error is returned if old password is incorrect. */
    oldValues[0] = password;
    oldValues[1] = NULL;
    delmod.mod_values = oldValues;

    addmod.mod_op   = LDAP_MOD_REPLACE;
    addmod.mod_type = "userPassword";
    newValues[0] = newPassword;
    newValues[1] = NULL;
    addmod.mod_values = newValues;

    modify[0] = &delmod;
    modify[1] = &addmod;
    modify[2] = NULL;

    /* Modify the attribute */
    rc= ldap_modify_ext_s( ld,         /* LDAP session handle */
                           loginDN,    /* the object to modify */
                           modify,     /* array of LDAPMod structures */
                           NULL,       /* server controls */
                           NULL);      /* client controls */

    if ( rc != LDAP_SUCCESS )
    {
        printf("\n\tldap_modify_ext_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s( ld );
        return(1);
    }
    printf("\n\tPassword modified for: %s\n", loginDN );

    ldap_unbind_s( ld );

    return( 0 );
}

