/* $Novell: /cm/ldap/src/cldap/samples/nameandoid.c,v 1.5 2002/01/08 22:08:59 dsteck Exp $ */
/******************************************************************************

   Copyright 2001 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   nameandoid.c 
***************************************************************************
   Description:  nameandoid.c shows how to get an oid from its name or get 
                 a name from its oid. A single Novell eDirectory syntax may
                 have multiple matches of LDAP syntaxes. nameandoid.c does
                 a table lookup. Extensions and controls supported by Novell
                 eDirectory are also included in the table.
 
                 The names and oids are defined in:
                   1. LDAP RFC 2252: Lightweight Directory Access Protocol(v3)
                   2. Internet Draft: draft-sermersheim-nds-ldap-schema-**.txt
                   3. Internet Draft: draft-khan-ldapext-replica-mgmt-**.txt
                   4. Internet Draft: draft-rharrision-lburp-**.txt
                   5. Internet Draft: draft-smith-psearch-ldap-**.txt
                   6. Internet Draft: draft-ietf-ldapext-ldapv3-vlv-**.txt


   sample runs:  $ nameandoid SYN_STREAM
                 
                   oid: 1.3.6.1.4.1.1466.115.121.1.5
                   Descriptive name: Binary
                   Novell eDirectory syntax name: SYN_STREAM
                   Reference: RFC 2252: Lightweight Directory Access Protocol(v3)                  

                 $ nameandoid "Directory String"
 
                   oid: 1.3.6.1.4.1.1466.115.121.1.15
                   Descriptive name: Directory String
                   Novell eDirectory syntax name: SYN_CI_STRING
                   Reference: RFC 2252: Lightweight Directory Access Protocol(v3)

                 $ nameandoid "Add Replica Request"

                   oid: 2.16.840.1.113719.1.27.100.7
                   Descriptive name: Add Replica Request
                   Reference: Internet Draft: draft-khan-ldapext-replica-mgmt-**.txt

                 $ nameandoid "Persistent Search"

                   oid: 2.16.840.1.113730.3.4.3
                   Descriptive name: Persistent Search
                   Reference: Internet Draft: draft-ietf-ldapext-psearch-**.txt

                 $ namwandoid "1.3.6.1.4.1.1466.115.121.1.1"

                   oid: 1.3.6.1.4.1.1466.115.121.1.1
                   Descriptive name: ACI Item
                   Novell eDirectory syntax name: SYN_OCTET_STRING
                   Reference: RFC 2252: Lightweight Directory Access Protocol(v3)

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <ldap_oids.h>

void printNameAndOID(int);

static char usage[] =
"\n Usage:   nameandoid <name/OID> "
"\n Example: nameandoid \"SYN_STREAM\""
"\n or"
"\n Example: nameandoid \"Directory String\""
"\n or"
"\n Example: nameandoid \"ADD_REPLICA_REQ\""
"\n or"
"\n Example: nameandoid \"Persistent Search\""
"\n or"
"\n Example: nameandoid \"1.3.6.1.4.1.1466.115.121.1.1\""
"\n"
"\n Note:    Name can be a discriptivr name, a Novell eDirectory"
"\n          syntax name, a control name, or a extension name"
"\n          supported by Novell eDirectory\n";

int main( int argc, char **argv)
{
    int  i, len, type, found = 0;
    char *value;

    if (argc != 2) {
        printf("%s", usage);
        return(1);
    }

    value = argv[1];
    len = strlen(value);

    if (len == 0) {
        printf("%s", usage);
        return(1);
    }
    else {      
        for (i = 0; i < len; i++) {
            if ( !isdigit(value[i]) && (value[i]!='.')) {
                type = 0;
                break;
            }
            else
                type = 1;                       
        }
        
        if ( type == 0) {     /* from name to oid */
            for ( i = 0; i < TOTAL_LEN; i++ ) {
                if (  (strcmp( value, namesandoids[i][1]) == 0) 
                    ||(strcmp( value, namesandoids[i][2]) == 0)) {
                        found = 1;
                        printNameAndOID(i);
                }
            }
        }
        else { /* from oid to name */
            for ( i = 0; i < TOTAL_LEN; i++ ) {
                if ( strcmp( value, namesandoids[i][3]) == 0 ) {
                    found = 1;
                    printNameAndOID(i);
                }
            }
        }            

        if(!found)      
            printf("Unknown %s%s\n", type? "oid: ":"name: ", value);
    }   

    return( 0 );
}

void printNameAndOID( int index) {        
    char *dName = namesandoids[index][1];
    char *sName = namesandoids[index][2];    
    char *oid   = namesandoids[index][3];
    char *ref   = namesandoids[index][4];
        
    if (*oid)   printf("oid: %s\n", oid);
    if (*dName) printf("Descriptive name: %s\n", dName);
    if (*sName)  printf("Novell eDirectory syntax name: %s\n", sName );           
    if (*ref)   printf("Reference: %s\n", ref );    
    printf("\n");
}

