/* $Novell: /cm/ldap/src/cldap/samples/sslbind.c,v 1.11 2001/11/27 20:10:38 fzhao Exp $ */
/*************************************************************************

   Copyright 1999, 2000, 2001 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
    sslbind.c
***************************************************************************
   Description: Demonstrates connecting to a to an ldap server using SSL
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ldap_ssl.h>

static char usage[] =
"\n Usage:   sslbind <host name> <port number> <login dn> <password>"
"\n\t<cert file> <file type> \n"
"  host name      = ldap server name or IP address\n"
"  port number    = port to use - 636 is ldap ssl default\n" 
"  login dn       = user name to login as\n"
"  password       = user password\n"
"  cert file      = trusted root certificate file\n"
"  file type      = DER=der encoded file, B64=b64 encoded file\n"
"\n Example: sslbind Acme.com 636 cn=admin,o=Acme secret myKey.der DER\n";

int main(int argc, char *argv[])
{

    int  version, rc, ldapPort;
    char *ldapHost;
    char *loginDN;
    char *password;
    char *certFile;
    char *fileType;
    int   fileEncoding;

    LDAP           *ld;
    LDAPMessage    *resultBuf = NULL;
    LDAPMessage    *entry = NULL;
    char           *dn;
       

    if (argc != 7)
    {
        printf("%s", usage);
        exit(1);
    }

    ldapHost     = argv[1];
    ldapPort     = atoi(argv[2]);
    loginDN      = argv[3];
    password     = argv[4];
    certFile     = argv[5];
    fileType     = argv[6];
   

    /* 
     * Set LDAP version to 3. 
     */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version);

    /* 
     * Validate the fileType argument
     */
    if ( (0 == strcmp(fileType, "DER")) || (0 == strcmp(fileType, "der")))
       fileEncoding = LDAPSSL_CERT_FILETYPE_DER;
    else if ((0 == strcmp(fileType, "B64")) ||(0 == strcmp(fileType, "b64")))
      fileEncoding = LDAPSSL_CERT_FILETYPE_B64;
    else
    {
       printf("Invalid certificate file type.\n");
       printf("%s", usage);
       exit(1);
    }


    /* 
     * Initialize the ssl library.  The first parameter of 
     * ldapssl_client_init is a certificate file.  However, when used
     * the file must be a DER encoded file.  A the API, 
     * ldapssl_add_trusted_cert accepts both DER and B64 encoded files.
     *
     * ldapssl_client_init is an application level initialization not a 
     * thread level initilization and should be done once.
     */
    rc = ldapssl_client_init(   NULL,       /* DER encoded cert file  */ 
                                NULL );     /* reserved, use NULL */

    if (rc != LDAP_SUCCESS)
    {
        printf("ldapssl_client_init error: %d\n", rc);
        exit(1);
    }


    /* 
     * Add the trusted root certificate file.  Multiple trusted root 
     * certificates can be added with ldapssl_add_trusted_cert.  Applications
     * can use this functionality to specify groups of trusted servers.
     */   
    rc = ldapssl_add_trusted_cert(certFile, fileEncoding);
    if (rc != LDAP_SUCCESS)
    {
        printf("ldapssl_add_trusted_cert error: %d\n", rc);
        ldapssl_client_deinit();
        exit(1);
    }


    /* 
     * create a LDAP session handle that is enabled for ssl connection
     */
    ld = ldapssl_init(  ldapHost,       /* host name */
                        ldapPort,       /* port number */
                        1 );            /* 0- clear text, 1-enable for ssl */

    if (ld == NULL )
    {
        printf("ldapssl_init error\n" );
        ldapssl_client_deinit();
        exit(1);
    }

    rc = ldap_simple_bind_s( ld, loginDN, password);
   
    if (rc != LDAP_SUCCESS )
    {
        printf("ldap_simple_bind_s error: %d, %s\n", rc, ldap_err2string( rc ));
        ldap_unbind_s( ld );
        ldapssl_client_deinit();
        exit(1);
    }
    

    printf("SSL bind successful - performing search\n");
      

    /* 
     * do a simple search just to verify things worked
     */
    rc = ldap_search_s( ld, "" , LDAP_SCOPE_ONELEVEL, "objectClass=*",
                        NULL, 0, &resultBuf);

    if( rc == LDAP_SUCCESS)
    {
       entry = ldap_first_entry(ld, resultBuf);

       while (entry != NULL)
       {
          dn = ldap_get_dn(ld, entry);
          printf("\tObject: %s\n", dn);

          ldap_memfree(dn);

          entry = ldap_next_entry(ld, entry);
       }

       ldap_msgfree( resultBuf );
    }
    else
    {
       printf("ldap_search_s error: %d, %s\n", rc, ldap_err2string(rc));
    }

    ldap_unbind_s( ld );

    /* 
     * Uninitialize the LDAP ssl library
     */
    ldapssl_client_deinit();

    return 0;
}

