/* $Novell: /cm/ldap/src/cldap/samples/verpass.c,v 1.5 2001/12/19 22:19:56 dsteck Exp $ */
/*************************************************************************

   Copyright 1999, 2000 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   verpass.c   
***************************************************************************
   Description: The verpass.c sample verifies that a password is correct 
                for the given object.
                
                We simply compare the password with the "userPassword"
                attribute of the object using the LDAP compare function.
                
                Verifying the password of an object is a much faster
                operation than binding to the object.  However, it
                does not give you the rights or identity of that
                object like the bind does.
                
                Note also that successfully comparing a password does not 
                necessarily mean that the object could successfully bind
                with that password.  The password may be expired, or the 
                account disabled, or time restrictions may prevent the user 
                from binding.
                
                eDirectory version 8.5 or higher is required to run this sample.
                                   
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ldap.h>

static char usage[] =
"\n Usage:   verpass <host name> <port number> <login dn> <password>"
"\n          <object dn> <test password>\n" 
"\n Example: verpass Acme.com 389 cn=admin,o=Acme secret"
"\n          cn=james,ou=Sales,o=Acme testpass\n";

int main( int argc, char **argv)
{ 

    LDAP        *ld;
    int         version, ldapPort, rc; 
    char        *ldapHost, *loginDN, *password, *objectDN, *testPassword;

    if (argc != 7)
    {
        printf("%s", usage);
        return(1);
    }

    ldapHost     =   argv[1];
    ldapPort     =   atoi(argv[2]);
    loginDN      =   argv[3];
    password     =   argv[4];
    objectDN     =   argv[5];
    testPassword =   argv[6];

    /* Set LDAP version to 3 */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version);

    /* Initialize the LDAP session */
    if (( ld = ldap_init( ldapHost, ldapPort )) == NULL)
    {
        printf ( "\n\tLDAP session initialization failed\n");
        return( 1 );
    }
    printf ( "\n\tLDAP session initialized\n");

    /* Bind to the server */
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if (rc != LDAP_SUCCESS )
    {
        printf("ldap_simple_bind_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s ( ld );
        return( 1 );
    }
    printf("\n\tBind successful\n");


    /* 
     * Compare the test password with the userPassword attribute 
     * of the object.
     */
    rc= ldap_compare_s( ld,              /* LDAP session handle */
                        objectDN,        /* the object to verify password on */
                        "userPassword",  /* attribute to compare */
                        testPassword);   /* the password to verify */

    if ( rc == LDAP_COMPARE_TRUE)
        printf("\n\tThe password is correct.\n");
    else if ( rc == LDAP_COMPARE_FALSE )
        printf("\n\tThe password is not correct.\n");
    else
    {
        printf("\n\tldap_compare_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s( ld );
        return(1);
    }

    ldap_unbind_s( ld );

    return( 0 );
}

