/* $Novell: /ldap/src/cldap/samples/vlvcntl.c,v 1.9 2001/07/30 18:43:17 dsteck Exp $ */
/*************************************************************************

   Copyright 1999, 2000 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   vlvcntl.c 
***************************************************************************
   Description: The vlvcntl.c sets up a server-side sort control and 
                a virtual list view control, searches 
                the directory, and prints the results.
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ldap.h>

static char usage[] =
"\n Usage:   vlvcntl <host name> <port number> <login dn> <password>"
"\n          <search base>\n" 
"\n Example: vlvcntl acme.com 389 cn=admin,o=acme secret ou=sales,o=acme\n";

int main( int argc, char **argv)
{
    int   ldapPort;
    char  *ldapHost, *loginDN, *password, *searchBase;
    LDAP  *ld;
    int   rc;
    int   version;
    int   err;
    unsigned long err1;
    char  *dn = NULL;
    char  **values = NULL;
    LDAPSortKey **sortkeylist = NULL;
    LDAPMessage *searchResult = NULL;
    LDAPMessage *entry = NULL;
    LDAPControl *serverctrls[3] = {NULL, NULL, NULL};  /* Null-terminated */
    LDAPControl **resultControls = NULL;
    LDAPVLVInfo vlvInfo;

    /*
     * Sort control parameters.
     */
    char  *sortAttr = "sn";                        /* Attribute to sort on */
    char  *attrs[] = {"givenname","sn", NULL };    /* Attributes to return */       
    int   attrsonly = 0;                           /* Return attrs and values */
    int   isCritical = 1;                          /* Control is critical */
    char  *searchFilter = "(sn=*)";                /* Search filter to use */
    /*
     * VLV control parameters.  If typedown is NULL, index mode is used.
     * The values below should return 4 entries, entry numbers 2 through 5, 
     * with the 3rd entry as the target.
     */
    unsigned long  target = 3;            /* Retrieve 3rd entry in list */
    unsigned long  before_count = 1;      /* Get one entry before target */
    unsigned long  after_count = 2;       /* Get two entries after the target */
    unsigned long  total_count = 0;       /* Estimate of list size. Init to 0 */
    struct berval  *pContext = NULL;      /* Initially set to NULL */

    if (argc != 6)
    {
        printf("%s", usage);
        return ( 1 );
    }
    
    ldapHost   = argv[1];
    ldapPort   = atoi(argv[2]);
    loginDN    = argv[3];
    password   = argv[4];
    searchBase = argv[5];
    
    printf("\nSort key:  %s\n", sortAttr);
    printf("Target index:  %d\nBefore count:  %d\nAfter count:   %d\n",
           target, before_count, after_count);
    printf("Client's total_count estimate:  %d\n", total_count);
    
     
    /* 
     * Set LDAP version 3 protocol 
     */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version);
     
    /* 
     * Initialize an LDAP session
     */
    ld = ldap_init( ldapHost, ldapPort );
    if (NULL == ld)
    {
        printf( "\n\tLDAP session initialization failed\n");
        return ( 1 );
    }        
    
    /* 
     * Authenticate to the server 
     */
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_simple_bind_s: %s\n", ldap_err2string(rc));
        return ( rc );
    }
    
    /* 
     * Create the sort key structure specifying the attribute to sort on. 
     */
    rc = ldap_create_sort_keylist(&sortkeylist, sortAttr);
    if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_create_sort_keylist: %s\n", ldap_err2string(rc));
        goto cleanup;
    }

    /* 
     * Create the sort control. sort control
     * must comes before vlv control
     */
    rc = ldap_create_sort_control(
            ld,                         /* Session handle */
            sortkeylist,                /* Array of sortkey structs */
            isCritical,                 /* Criticality of this control */
            &serverctrls[0]);           /* The returned control structure */

    if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_create_sort_control: %s\n",ldap_err2string(rc));
        goto cleanup;
    }
    
    /*
     * Create the VLV control.
     */
     vlvInfo.ldvlv_version = 1;                 /* Version of this structure */
     vlvInfo.ldvlv_attrvalue = NULL;            /* Typedown value*/
     vlvInfo.ldvlv_offset = target;             /* Index of desired entry */
     vlvInfo.ldvlv_before_count = before_count; /* # of entries before target */
     vlvInfo.ldvlv_after_count = after_count;   /* # of entries after target */
     vlvInfo.ldvlv_count = total_count;         /* Client's est. of list size */
     vlvInfo.ldvlv_context = pContext;          /* Result of previous control */
     vlvInfo.ldvlv_extradata = NULL;            /* Not used */
    
     rc = ldap_create_vlv_control(
            ld,                         /* Session handle */
            &vlvInfo,                   /* VLV information structure */
            &serverctrls[1]);           /* The returned control structure */
    
     if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_create_vlv_control: %s\n",ldap_err2string(rc));
        goto cleanup;
    }

    /*
     * Perform the search with server-side sorting and vlv control.
     */
    printf("\nPerforming search with sort and VLV controls...\n");
    rc = ldap_search_ext_s( 
            ld,                            /* Session handle */
            searchBase,                    /* Container to search */
            LDAP_SCOPE_ONELEVEL,           /* Search this container only */
            searchFilter,                  /* Search filter to use */
            attrs,                         /* Attributes to return */
            attrsonly,                     /* Return attrs and values */
            serverctrls,                   /* Server controls */
            NULL,                          /* No client controls */
            LDAP_NO_LIMIT,                 /* No time limit */
            LDAP_NO_LIMIT,                 /* No size limit */ 
            &searchResult);                /* Returned results */

    if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_search_ext_s: %s\n", ldap_err2string(rc));
        goto cleanup;
    }

    /*
     * Parse the search results to obtain the return controls.
     */
    rc = ldap_parse_result(
            ld,                         /* Session handle */
            searchResult,               /* Results from the search operation */
            &err,                       /* Error code of the search operation */
            NULL,                       /* Matched DN */
            NULL,                       /* Error message */
            NULL,                       /* Referrals */ 
            &resultControls,            /* Returned controls */
            0);                         /* Don't free the searchResult */
    
    if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_parse_result: %s\n", ldap_err2string(rc));
        goto cleanup;
    }

    /* 
     * Parse the returned sort control.
     */
    rc = ldap_parse_sort_control(
            ld,                     /* Session handle */                   
            resultControls,         /* Results from the search operation */
            &err1,                   /* Error code for the vlv control */
            NULL);                  /* Attribute involved in the error */

    if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_parse_sort_control: %s\n", ldap_err2string(rc));
        goto cleanup;
    }
    if (err)
        printf("*** Sort control error: %s\n\n", ldap_err2string((int)err1));


    /* 
     * Parse the returned VLV control.
     */
    rc = ldap_parse_vlv_control(
            ld,                     /* Session handle */
            resultControls,         /* Results from the search operation */
            &target,                /* Server's estimate of target position */
            &total_count,           /* Server's estimate of total count */
            &pContext,              /* Context returned for this search */
            &err);                  /* Error code for the vlv control */

    if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_parse_vlv_control: %s\n", ldap_err2string(rc));
        goto cleanup;
    }
    if (err)
        printf("*** VLV control error: %s\n\n", ldap_err2string(err));
    else
    {
        printf("\nParse VLV control results:\n");
        printf("  Server's target index: %d\n", target);
        printf("  Server's total count: %d\n\n", total_count);
    }

    /* 
     * For each entry print out dn, sn, and givenname.
     * Only print first value if there are multiple values.
     */
    printf("Entry <dn>:   <sn>, <given name>\n\n");
    for ( entry = ldap_first_entry( ld, searchResult ); 
          entry != NULL;
          entry = ldap_next_entry( ld, entry ) ) 
    {
        dn = ldap_get_dn( ld, entry );
        if (dn)
        {
            printf("Object %s:\t",dn);
            ldap_memfree(dn);
        }

        if ((values = ldap_get_values( ld, entry, "sn")) != NULL ) 
        {
            if ( values[0] != NULL )    
                printf("%s,  ", values[0] );
            ldap_value_free( values );
        }
   
        if ((values = ldap_get_values( ld, entry, "givenname")) != NULL ) 
        {
           if ( values[0] != NULL ) 
               printf("%s", values[0] );
           ldap_value_free( values );
        }
        printf("\n" );
    }
            
cleanup:
    ber_bvfree( pContext );
    ldap_msgfree( searchResult );
    ldap_control_free( serverctrls[0] );    /* Free the sort control */
    ldap_control_free( serverctrls[1] );    /* Free the vlv control */
    ldap_controls_free( resultControls );
    ldap_free_sort_keylist( sortkeylist );
    ldap_unbind_s( ld );
   
    return( rc );
}
