/*								 
 * $Novell: /cm/ldap/src/cldap/nldapx/extensions/sdkinc/common/events.h,v 1.2 2001/12/18 22:07:55 vtag Exp $
 ******************************************************************************
 * Copyright (C) 1999, 2000 Novell, Inc. All Rights Reserved.
 * 
 * THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
 * TREATIES. USE, MODIFICATION, AND REDISTRIBUTION OF THIS WORK IS SUBJECT
 * TO VERSION 2.0.7 OF THE OPENLDAP PUBLIC LICENSE, A COPY OF WHICH IS
 * AVAILABLE AT HTTP://WWW.OPENLDAP.ORG/LICENSE.HTML OR IN THE FILE "LICENSE"
 * IN THE TOP-LEVEL DIRECTORY OF THE DISTRIBUTION. ANY USE OR EXPLOITATION
 * OF THIS WORK OTHER THAN AS AUTHORIZED IN VERSION 2.0.7 OF THE OPENLDAP
 * PUBLIC LICENSE, OR OTHER PRIOR WRITTEN CONSENT FROM NOVELL, COULD SUBJECT
 * THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY. 
 ******************************************************************************
 */
#ifndef EVENTS_H
#define EVENTS_H

#include <lber_types.h>

/* The following constants are used to set the event status */
#define EVT_STATUS_ALL        0
#define EVT_STATUS_SUCCESS    1
#define EVT_STATUS_FAILURE    2

typedef struct {
	int eventType;
	int eventStatus;
} NDSEventSpecifier;

#define EVT_INVALID 						0
#define EVT_CREATE_ENTRY					1	/* data is EVT_EntryInfo */
#define EVT_DELETE_ENTRY					2	/* data is EVT_EntryInfo */
#define EVT_RENAME_ENTRY					3	/* data is EVT_EntryInfo */
#define EVT_MOVE_SOURCE_ENTRY				4	/* data is EVT_EntryInfo */
#define EVT_ADD_VALUE						5	/* data is EVT_ValueInfo */
#define EVT_DELETE_VALUE					6	/* data is EVT_ValueInfo */
#define EVT_CLOSE_STREAM					7	/* data is EVT_ValueInfo */
#define EVT_DELETE_ATTRIBUTE				8	/* data is EVT_ValueInfo */
#define EVT_SET_BINDERY_CONTEXT 			9	/* no data */
#define EVT_CREATE_BINDERY_OBJECT			10	/* data is EVT_BinderyObjectInfo */
#define EVT_DELETE_BINDERY_OBJECT			11	/* data is EVT_BinderyObjectInfo */
#define EVT_CHECK_SEV						12	/* data is EVT_SEVInfo */
#define EVT_UPDATE_SEV						13	/* no data */
#define EVT_MOVE_DEST_ENTRY 				14	/* data is EVT_EntryInfo */
#define EVT_DELETE_UNUSED_EXTREF			15	/* data is EVT_EntryInfo */
#define EVT_OBSOLETE_EVT_TRACE				16	/* data is EVT_ModuleStatea */
#define EVT_REMOTE_SERVER_DOWN				17	/* data is NetAddress */
#define EVT_NCP_RETRY_EXPENDED				18	/* data is NetAddress */
#define EVT_REMOTE_CONN_CLEARED 			19	/* data is NetAddress */
#define EVT_PARTITION_OPERATION_EVENT		20	/* data is DSEEventData; used by SCALE */
#define EVT_CHANGE_MODULE_STATE				21	/* data is EVT_ModuleState */


#define EVT_AGENT_OPEN_LOCAL				53	/* EVT_EventData */
#define EVT_AGENT_CLOSE_LOCAL				54	/* EVT_EventData */
#define EVT_DS_ERR_VIA_BINDERY				55	/* EVT_EventData */
#define EVT_DSA_BAD_VERB					56	/* EVT_EventData */
#define EVT_DSA_REQUEST_START				57	/* EVT_EventData */
#define EVT_DSA_REQUEST_END 				58	/* EVT_EventData */
#define EVT_MOVE_SUBTREE					59	/* EVT_EventData */
#define EVT_NO_REPLICA_PTR					60	/* EVT_EventData */
#define EVT_SYNC_IN_END 					61	/* EVT_EventData */
#define EVT_BKLINK_SEV						62	/* EVT_EventData */
#define EVT_BKLINK_OPERATOR 				63	/* EVT_EventData */
#define EVT_DELETE_SUBTREE					64	/* EVT_EventData */
#define EVT_SET_NEW_MASTER					65	/* EVT_EventData */
#define EVT_PART_STATE_CHG_REQ				66	/* EVT_EventData */
#define EVT_REFERRAL						67	/* EVT_EventData */
#define EVT_UPDATE_CLASS_DEF				68	/* EVT_EventData */
#define EVT_UPDATE_ATTR_DEF 				69	/* EVT_EventData */
#define EVT_LOST_ENTRY						70	/* EVT_EventData */
#define EVT_PURGE_ENTRY_FAIL				71	/* EVT_EventData */
#define EVT_PURGE_START 					72	/* EVT_EventData */
#define EVT_PURGE_END						73	/* EVT_EventData */
#define EVT_FLAT_CLEANER_END				74	/* EVT_EventData */
#define EVT_ONE_REPLICA 					75	/* EVT_EventData */
#define EVT_LIMBER_DONE 					76	/* EVT_EventData */
#define EVT_SPLIT_DONE						77	/* EVT_EventData */
#define EVT_SYNC_SVR_OUT_START				78	/* EVT_EventData */
#define EVT_SYNC_SVR_OUT_END				79	/* EVT_EventData */
#define EVT_SYNC_PART_START 				80	/* EVT_EventData */
#define EVT_SYNC_PART_END					81	/* EVT_EventData */
#define EVT_MOVE_TREE_START 				82	/* EVT_EventData */
#define EVT_MOVE_TREE_END					83	/* EVT_EventData */
#define EVT_RECERT_PUB_KEY					84	/* EVT_EventData */
#define EVT_GEN_CA_KEYS 					85	/* EVT_EventData */
#define EVT_JOIN_DONE						86	/* EVT_EventData */
#define EVT_PARTITION_LOCKED				87	/* EVT_EventData */
#define EVT_PARTITION_UNLOCKED				88	/* EVT_EventData */
#define EVT_SCHEMA_SYNC 					89	/* EVT_EventData */
#define EVT_NAME_COLLISION					90	/* EVT_EventData */
#define EVT_NLM_LOADED						91  /* EVT_EventData */
/*#define EVT_PARTITION_EVENT			    92*/  /* ** Uses DSEPartitionData structure */
#define EVT_SKULKER_EVENT					93  /* EVT_SkulkData */
#define EVT_LUMBER_DONE 					94	/* no data */
#define EVT_BACKLINK_PROC_DONE				95	/* no data */
#define EVT_SERVER_RENAME					96	/* EVT_EventData */
#define EVT_SYNTHETIC_TIME					97	/* EVT_EventData */
#define EVT_SERVER_ADDRESS_CHANGE			98	/* no data */
#define EVT_DSA_READ						99	/* EVT_EventData */
#define EVT_LOGIN							100 /* EVT_EventData */
#define EVT_CHGPASS 						101 /* EVT_EventData */
#define EVT_LOGOUT							102 /* EVT_EventData */
#define EVT_ADD_REPLICA 					103 /* EVT_EventData */
#define EVT_REMOVE_REPLICA					104 /* EVT_EventData */
#define EVT_SPLIT_PARTITION 				105 /* EVT_EventData */
#define EVT_JOIN_PARTITIONS 				106 /* EVT_EventData */
#define EVT_CHANGE_REPLICA_TYPE 			107 /* EVT_EventData */
#define EVT_REMOVE_ENTRY					108 /* EVT_EventData */
#define EVT_ABORT_PARTITION_OP				109 /* EVT_EventData */
#define EVT_RECV_REPLICA_UPDATES			110 /* EVT_EventData */
#define EVT_REPAIR_TIME_STAMPS				111 /* EVT_EventData */
#define EVT_SEND_REPLICA_UPDATES			112 /* EVT_EventData */
#define EVT_VERIFY_PASS 					113 /* EVT_EventData */
#define EVT_BACKUP_ENTRY					114 /* EVT_EventData */
#define EVT_RESTORE_ENTRY					115 /* EVT_EventDatan */
#define EVT_DEFINE_ATTR_DEF 				116 /* EVT_EventData */
#define EVT_REMOVE_ATTR_DEF 				117 /* EVT_EventData */
#define EVT_REMOVE_CLASS_DEF				118 /* EVT_EventData */
#define EVT_DEFINE_CLASS_DEF				119 /* EVT_EventData */
#define EVT_MODIFY_CLASS_DEF				120 /* EVT_EventData */
#define EVT_RESET_DS_COUNTERS				121 /* EVT_EventData */
#define EVT_REMOVE_ENTRY_DIR				122 /* EVT_EventData */
#define EVT_COMPARE_ATTR_VALUE				123 /* EVT_EventData */
#define EVT_STREAM		 					124 /* EVT_EventData */
#define EVT_LIST_SUBORDINATES				125 /* EVT_EventData */
#define EVT_LIST_CONT_CLASSES				126 /* EVT_EventData */
#define EVT_INSPECT_ENTRY					127 /* EVT_EventData */
#define EVT_RESEND_ENTRY					128 /* EVT_EventData */
#define EVT_MUTATE_ENTRY					129 /* EVT_EventData */
#define EVT_MERGE_ENTRIES					130 /* EVT_EventData */
#define EVT_MERGE_TREE						131 /* EVT_EventData */
#define EVT_CREATE_SUBREF					132 /* EVT_EventData */
#define EVT_LIST_PARTITIONS 				133 /* EVT_EventData */
#define EVT_READ_ATTR						134 /* EVT_EventData */
#define EVT_READ_REFERENCES 				135 /* EVT_EventData */
#define EVT_UPDATE_REPLICA					136 /* EVT_EventData */
#define EVT_START_UPDATE_REPLICA			137 /* EVT_EventData */
#define EVT_END_UPDATE_REPLICA				138 /* EVT_EventData */
#define EVT_SYNC_PARTITION					139 /* EVT_EventData */
#define EVT_SYNC_SCHEMA 					140 /* EVT_EventData */
#define EVT_CREATE_BACKLINK 				141 /* EVT_EventData */
#define EVT_CHECK_CONSOLE_OPERATOR			142 /* EVT_EventData */
#define EVT_CHANGE_TREE_NAME				143 /* EVT_EventData */
#define EVT_START_JOIN						144 /* EVT_EventData */
#define EVT_ABORT_JOIN						145 /* EVT_EventData */
#define EVT_UPDATE_SCHEMA					146 /* EVT_EventData */
#define EVT_START_UPDATE_SCHEMA 			147 /* EVT_EventData */
#define EVT_END_UPDATE_SCHEMA				148 /* EVT_EventData */
#define EVT_MOVE_TREE						149 /* EVT_EventData */
#define EVT_RELOAD_DS						150 /* EVT_EventData */
#define EVT_ADD_PROPERTY					151 /* EVT_EventData */
#define EVT_DELETE_PROPERTY 				152 /* EVT_EventData */
#define EVT_ADD_MEMBER						153 /* EVT_EventData */
#define EVT_DELETE_MEMBER					154 /* EVT_EventData */
#define EVT_CHANGE_PROP_SECURITY			155 /* EVT_EventData */
#define EVT_CHANGE_OBJ_SECURITY 			156 /* EVT_EventData */
#define EVT_READ_OBJ_INFO					157 /* EVT_EventData */
#define EVT_CONNECT_TO_ADDRESS				158 /* EVT_EventData */
#define EVT_SEARCH							159 /* EVT_EventData */
#define EVT_PARTITION_STATE_CHG 			160 /* EVT_EventData */
#define EVT_REMOVE_BACKLINK 				161 /* EVT_EventData */
#define EVT_LOW_LEVEL_JOIN					162 /* EVT_EventData */
#define EVT_CREATE_NAMEBASE 				163 /* EVT_EventData */
#define EVT_CHANGE_SECURITY_EQUALS			164 /* EVT_EventData */
#define EVT_DB_OBSOLETEGUID					165 /* EVT_EventData */
/*#define EVT_DB_NCPENG						166*/
#define EVT_CRC_FAILURE 					167 /* EVT_EventData */
#define EVT_ADD_ENTRY						168 /* EVT_EventData */
#define EVT_MODIFY_ENTRY					169 /* EVT_EventData */
/*#define EVT_DB_OBSOLETEFORWARDLINK */		
#define EVT_OPEN_BINDERY					171 /* EVT_EventData */
#define EVT_CLOSE_BINDERY					172 /* EVT_EventData */
#define EVT_CHANGE_CONN_STATE				173 /* EVT_ChangeConnState */
#define EVT_NEW_SCHEMA_EPOCH				174 /* EVT_EventData */
#define EVT_MODIFY_RDN			        	178 /* EVT_EventData */
#define EVT_ORPHAN_PARTITION        		180 /* EVT_EventData */
#define EVT_ENTRYID_SWAP					181 /* EVT_EventData */

#define EVT_INSIDE_NCP_REQUEST				182 /* no data */
#define EVT_LOW_LEVEL_SPLIT					185 /* EVT_EventData */
#define EVT_END_NAMEBASE_TRANSACTION 		187 /* no data */
#define EVT_ALLOW_LOGIN						188 /* EVT_EventData */

#define EVT_WTM_NDS_BACKLINKS				191 /* DSEWtmInfo */
#define EVT_WTM_NDS_SCHEMA_SYNC				192 /* DSEWtmInfo */
#define EVT_WTM_NDS_LIMBER					193 /* DSEWtmInfo */
#define EVT_WTM_NDS_LOGIN_RESTRICTIONS		194 /* DSEWtmInfo */
#define EVT_WTM_NDS_JANITOR					195 /* DSEWtmInfo */
#define EVT_WTM_NDS_OPEN_CONNECTION			196 /* DSEWtmOpenInfo */

#define EVT_LOCAL_REPLICA_CHANGE			197 /* d1-EVT_LRC_* opcode, d2-replicaRootID */
#define EVT_DB_DRL							198 /* reference link debug messages */
#define EVT_MOVE_ENTRY_SOURCE				199 /* d1-parentID, d2-dest parentID, d3-sourceID, uname-name */
#define EVT_MOVE_ENTRY_DEST					200 /* d1-parentID, d2-dest parentID, d3-sourceID, uname-newName */
#define EVT_NOTIFY_REF_CHANGE				201 /* d1-entryID used by obituary added to */
#define EVT_DB_ALLOC						202 /* uses DSEDebugInfo */
#define EVT_CONSOLE_OPERATION				203 /* d1-opCode - DSC_* flag */
#define EVT_DB_SERVER_PACKET				204 /* uses DSEDebugInfo */
#define EVT_START_DIB_CHECK 			   	205 /* RecMan DIB validation */
#define EVT_END_DIB_CHECK 			      	206 /* RecMan DIB validation, d1-error status */
#define EVT_REPLICA_IN_TRANSITION			208 /* d1-partition root ID, d2-last ID */
#define EVT_CHANGE_CONFIG_PARM				211 /* uses DSEChangeConfigParm */
#define EVT_COMPUTE_CONN_SEV_INLINE			212 /* EVT_ChangeConnState */
#define EVT_BEGIN_NAMEBASE_TRANSACTION		213 /* no data */
#define EVT_VR_DRIVER_STATE_CHANGE			215 /* used DSEEntryInfo */
#define EVT_REQ_UPDATE_SERVER_STATUS		216 /* Obtain current Status from specified Server */
#define EVT_DB_DIRXML_DRIVERS				217 /* uses DSEDebugInfo */
#define EVT_DB_NDSMON						218 /* ndsmon messages uses DSEDebugInfo */
#define EVT_CHANGE_SERVER_ADDRS				219 /* uses DSEChangeServerAddr - engine addresses changed */
#define EVT_DB_DNS							220 /* DNS messages - uses DSEDebugInfo */
#define EVT_DB_REPAIR						221 /* ds repair messages, uses dsedebuginfo */
#define EVT_DB_REPAIR_DEBUG					222 /* ds repair messages, uses dsedebuginfo */
#define EVT_REPAIR_PROGRESS					223 /* ds repair messages */
#define EVT_ITERATOR						224 /* d1-base object id, d2-scope, d3-uIteratorID, d4-infoType */
#define EVT_DB_SCHEMA_DETAIL				225	/* d1-(boolean value=)allProcessed, not in locks */
#define EVT_LOW_LEVEL_JOIN_BEGIN    		226 /* d1 parent partion root */
#define EVT_DB_IN_SYNC_DETAIL				227 /* uses DSEDebugInfo */
#define EVT_PRE_DELETE_ENTRY                228 /* data is EntryInfo This is a pre-delete event raised when an entry is about to be deleted - introduced for ldap psearch */
#define EVT_MAX_EVENTS	   					229


typedef struct 
{	unsigned int 	seconds; 
    unsigned short 	replicaNumber; 
    unsigned short 	event;
} EVT_TimeStamp;

typedef struct
{	char            *perpetratorDN; 
    char  		    *entryDN; 
    char  		    *parentDN; 
    char  		    *className; 
    unsigned int   	verb; 
    unsigned int   	flags;
	EVT_TimeStamp 	creationTime;
    char		    *newDN;
} EVT_EntryInfo;

/* EVT_EntryInfo flag definitions */
#define EVT_F_PARTITION_ROOT 	0x0001
#define EVT_F_EXTREF 		   	0x0002
#define EVT_F_ALIAS				0x0004

typedef struct
{	char  		  *perpetratorDN;
    unsigned int  verb; 
    char		  *entryDN; 
    char		  *attributeName;
    char		  *syntaxOID; 
    char		  *className;
	EVT_TimeStamp  timeStamp;
	unsigned 	  size;
	char  		  *value;
} EVT_ValueInfo;

typedef struct
{	char 		  *entryDN; 
    char		  *parentDN;
    unsigned int  type; 
    unsigned int  emuObjFlags;
    unsigned int  security;
	char 		  *name;
} EVT_BinderyObjectInfo;

typedef struct
{	char 		 *entryDN;
    unsigned int retryCount;
    unsigned int valueID;
	char 		 *valueDN;
	char 		 *referral;
} EVT_SEVInfo;

typedef struct
{   unsigned int		type;
	unsigned int		length;
	char		        *data;
} EVT_NetAddress;

typedef struct
{   unsigned int		entryID;
	unsigned int		attrID[2];
} EVT_SkulkData;

typedef struct
{   int 			connID;
	char*			entryDN;
	unsigned int    oldFlags;
    unsigned int	newFlags;
	int 			source;
	unsigned int	psid;
} EVT_ChangeConnState;


/*	Configuration parameter change data - data used for events generated when
 *	a system configuration parameter changes. The size of this structure is 
 * offsetof(EVT_ChangeConnState, name) + offset + size fields.
 *
 *	IDX - unique index value for configuration parameter node
 *	PIDX - unique index value for parent config parm node
 *	TYPE - node types defined below (KEY is ORed with other types)
 *	SIZE - size of data in bytes
 *	OFFSET --> DataPtr == &NAME[OFFSET];
 *	NAME - configuration parameter component name
 *
 *	The following definitions are key types, TYPE_KEY and TYPE_DELETED are
 *	flags that may be set on any of the others to indicate it is a non-leaf 
 *	node, or that it has been deleted. You will never read deleted keys from
 *	ACS, but you may get event notifications of key deletes. TYPE_NULL is
 * set on nodes that have no data assigned to them. Use TYPE_NULL to add a 
 * parameter with no data.
 */
#define EVT_CFG_TYPE_KEY		0x80000000
#define EVT_CFG_TYPE_DELETED	0x40000000
#define EVT_CFG_TYPE_RDONLY	0x20000000

#define EVT_CFG_TYPE_NULL		0
#define EVT_CFG_TYPE_BINARY	1
#define EVT_CFG_TYPE_unsigned int	2
#define EVT_CFG_TYPE_STRING	3
#define EVT_CFG_TYPE_UNICODE	4
#define EVT_CFG_TYPE_BOOLEAN	5

#define EVT_CFG_TYPE_ATTRS		0xF0000000
#define EVT_CFG_TYPE_CLASS		(~EVT_CFG_TYPE_ATTRS)

typedef struct EVT_ChangeConfigParm
{   int 		idx;
    int		    pidx;
    int		    type;
	unsigned 	size;
    unsigned	offset;
	char 		*name;
} EVT_ChangeConfigParm;

#define EVT_MOD_HIDDEN	  		0x0001
#define EVT_MOD_SYSTEM	  		0x0002
#define EVT_MOD_ENGINE	  		0x0004
#define EVT_MOD_AUTOMATIC 		0x0008
#define EVT_MOD_FILE_MASK		0x00FF

#define EVT_MOD_POSTEVENT 		0x0100
#define EVT_MOD_AVAILABLE 		0x0200
#define EVT_MOD_LOADING	  		0x0400
#define EVT_MOD_MODIFY			0x0800
#define EVT_MOD_NEGATE_BIT		0x8000
#define EVT_MOD_EVENT_MASK		0xFF00

#define EVT_MAX_MODULE_NAME	32
#define EVT_MAX_MODULE_DESCR	128

typedef struct 
{   int 		connID;
	unsigned 	flags;
	int /*SAL_ModHandle*/ 	module, source;
	char 		name[EVT_MAX_MODULE_NAME];
	char 		description[EVT_MAX_MODULE_DESCR];
} EVT_ModuleState;


#define EVT_CSA_ADDRESS			0x0001
#define EVT_CSA_PSTACK			0x0002	
#define EVT_CSA_EVENT_MASK		0x00FF	

#define EVT_CSA_REMOVED			0x0100
#define EVT_CSA_POST_EVENT		0x0200
#define EVT_CSA_MOD_MASK		0xFF00	

#define EVT_MAX_PSTK_NAME		15

typedef struct EVT_ChangeServerAddr
{   unsigned 	flags;
    unsigned	psid;
	int/*SAL_ModHandle*/ 	source;
	int 		proto;
	char* /*SAL_AddrBuf*/ 	addr;
	char 		pstkname[EVT_MAX_PSTK_NAME + 1];
} EVT_ChangeServerAddr;


typedef struct 
{   unsigned int attrID; 
    unsigned int flags; 
    int    len; 
    void   *data;
} EVT_S1data;

typedef struct
{   unsigned int		count;
	EVT_S1data	value[1];
} EVT_Struct1;

typedef struct
{   unsigned int		dstime;
    unsigned int		milliseconds;	
	unsigned int		curProcess;	
	unsigned int		connID; 	
	unsigned int		verb;		
	char*       		perpetratorDN;	
    unsigned int		intValues[4];
    char*               strValues[4];
} EVT_EventData;

typedef struct
{
    unsigned int 	lastTime;
    unsigned int 	version;
    int 			checkEachNewOpen;
    int 			checkEachAlreadyOpen;
    unsigned int 	expirationInterval;
    unsigned int 	nextTime;
    unsigned int 	result;
    int 			send;
    unsigned int 	trafficTypeSubClass;
} EVT_WANTrafficMgrInfo;

typedef struct
{
	int 			version;
	unsigned int 	expirationInterval;
	int 			connIsAlreadyOpen;
	unsigned int 	lastAccessTime;
	unsigned int 	trafficType;
	char 			*trafficTypeName;
	int 			send;
	int 			dstAdrType;
	unsigned int	dstAdrSize;
	void 			*dstAdrData;
	char 			*localReferral;
	unsigned int 	lastSyncTime;
	unsigned int 	trafficTypeSubClass;
} EVT_WANTrafficMgrOpenInfo;


#endif /*EVENTS_H*/

